package benchmark.graphcoloring;

import static util.parser.InstanceTokens.AGENT;
import static util.parser.InstanceTokens.AGENTS;
import static util.parser.InstanceTokens.ARITY;
import static util.parser.InstanceTokens.CONFLICTS;
import static util.parser.InstanceTokens.CONSTRAINT;
import static util.parser.InstanceTokens.CONSTRAINTS;
import static util.parser.InstanceTokens.CONSTRAINT_MODEL;
import static util.parser.InstanceTokens.DESCRIPTION;
import static util.parser.InstanceTokens.DOMAIN;
import static util.parser.InstanceTokens.DOMAINS;
import static util.parser.InstanceTokens.DisCSP;
import static util.parser.InstanceTokens.FORMAT;
import static util.parser.InstanceTokens.ID;
import static util.parser.InstanceTokens.INSTANCE;
import static util.parser.InstanceTokens.INSTANCES;
import static util.parser.InstanceTokens.MODEL;
import static util.parser.InstanceTokens.NAME;
import static util.parser.InstanceTokens.NB_AGENTS;
import static util.parser.InstanceTokens.NB_CONSTRAINTS;
import static util.parser.InstanceTokens.NB_DOMAINS;
import static util.parser.InstanceTokens.NB_INSTANCES;
import static util.parser.InstanceTokens.NB_RELATIONS;
import static util.parser.InstanceTokens.NB_TUPLES;
import static util.parser.InstanceTokens.NB_VALUES;
import static util.parser.InstanceTokens.NB_VARIABLES;
import static util.parser.InstanceTokens.PRESENTATION;
import static util.parser.InstanceTokens.REFERENCE;
import static util.parser.InstanceTokens.RELATION;
import static util.parser.InstanceTokens.RELATIONS;
import static util.parser.InstanceTokens.SCOPE;
import static util.parser.InstanceTokens.SEMANTICS;
import static util.parser.InstanceTokens.SIMPLE;
import static util.parser.InstanceTokens.TKC;
import static util.parser.InstanceTokens.TYPE;
import static util.parser.InstanceTokens.VARIABLE;
import static util.parser.InstanceTokens.VARIABLES;
import static util.parser.InstanceTokens.XDisCSP_1_0;

import java.io.File;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Random;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

import dischoco.DisChoco;

import benchmark.ResultPrinter;
import gui.GuiSimulator;
import kernel.order.AgentOrdering;
import simulation.DisProblem;
import simulation.DisSolverSimulator;
import util.Commands;
import util.Logger;
import util.Tools;
import util.parser.InstanceParser;

public class URGCGenerator extends GuiSimulator {

	private int minNbNodes = 5; // number of variable in the problem
	private int stepNbNodes = 1; // number of variable in the problem
	private int maxNbNodes = 5; // number of variable in the problem
	private int minNbColours = 3; // the number of colours = domain size
	private int stepNbColours = 1; // the number of colours = domain size
	private int maxNbColours = 3; // the number of colours = domain size
	private int minDensity = 5;
	private int stepDensity = 1;
	private int maxDensity = 5;
	private int nbInstances = 50;
	private static final String NODE = "Node";
	private static final String DENSITY = "density";
	private static final String MIN_DENSITY = "MinDensity";
	private static final String MAX_DENSITY = "MaxDensity";
	private static final String STEP = "step";
	private static final String GC = "GC";
	private static final String NB_FOLDERS = "nbFolders";
	private static final String DIRECTORY = "directory";
	private static final String NB_NODES = "nbNodes";
	private static final String NB_COLORES = "nbColores";

	private File directory = new File("./" + GC + "/");
	public final static String fixKeys = DENSITY; // Commands.nbNodes,
	// Commands.nbColours,
	public final static int nbHeaderLine = 4;

	public URGCGenerator(int nbInstances) {
		if (!this.directory.isDirectory()) {
			this.directory.mkdir();
		}
		this.nbInstances = nbInstances;
	}

	public URGCGenerator(File directory, int nbInstances) {
		this.directory = directory;
		if (!this.directory.isDirectory()) {
			this.directory.mkdir();
		}
		this.nbInstances = nbInstances;
	}

	/**
	 * Constructor
	 * 
	 * @param directory
	 * @param nbNodes
	 * @param nbColors
	 * @param nbConstraint
	 * @param nbInstances
	 */
	public URGCGenerator(File directory, int nbNodes, int nbColours, int nbConstraint, int nbInstances) {
		this(directory, nbInstances);
		this.minNbNodes = nbNodes;
		this.minNbColours = nbColours;
		this.minDensity = (200 * nbConstraint) / (nbNodes * (nbNodes - 1));
	}

	public URGCGenerator(int nbNodes, int nbColours, double ConstDensity, int nbInstances) {
		this(nbInstances);
		this.setNbNodes(nbNodes, nbNodes, nbNodes);
		this.setNbCoulors(nbColours, nbColours, nbColours);
		this.setDensity(ConstDensity, ConstDensity, ConstDensity);
	}

	/**
	 * @param filePath
	 */
	public URGCGenerator() {
	}

	public void setDensity(double minDensity, double stepDensity, double maxDensity) {
		int min = (int) (minDensity * 100);
		int step = (int) (stepDensity * 100);
		int max = (int) (maxDensity * 100);
		this.setDensity(min, step, max);
	}

	public void setDensity(int minDensity, int stepDensity, int maxDensity) {
		this.minDensity = minDensity;
		this.stepDensity = stepDensity;
		this.maxDensity = maxDensity;
	}

	public void setNbNodes(int minNbNodes, int stepNbNodes, int maxNbNodes) {
		this.minNbNodes = minNbNodes;
		this.stepNbNodes = stepNbNodes;
		this.maxNbNodes = maxNbNodes;
	}

	public void setNbCoulors(int minNbColours, int stepNbColours, int maxNbColours) {
		this.minNbColours = minNbColours;
		this.stepNbColours = stepNbColours;
		this.maxNbColours = maxNbColours;
	}

	/**
	 * @param document
	 * @param file
	 */
	void save(Document document, String file) {
		try {
			XMLOutputter out = new XMLOutputter(Format.getPrettyFormat());
			out.output(document, new FileOutputStream(file));
			Process p = Runtime.getRuntime().exec("bzip2 " + file);
			p.waitFor();
			p.exitValue();
			p.destroy();
		} catch (Exception e) {}
	}

	/**
	 * @param subDirectory
	 * @param fileName
	 */
	void saveInDirectory(Document document, String fileName) {
		try {
			XMLOutputter out = new XMLOutputter(Format.getPrettyFormat());
			out.output(document, new FileOutputStream(this.directory.getPath() + File.separator + fileName));
		} catch (java.io.IOException e) {}
	}

	public Element getPresentation(int numInstance) {
		Element presentation = new Element(PRESENTATION);
		presentation.setAttribute(NAME, GC + numInstance);
		presentation.setAttribute(MODEL, SIMPLE);
		presentation.setAttribute(CONSTRAINT_MODEL, TKC);
		presentation.setAttribute(FORMAT, XDisCSP_1_0);
		presentation.setAttribute(TYPE, DisCSP);
		return presentation;
	}

	public Element getAgents(int nbNodes) {
		Element agents = new Element(AGENTS);
		agents.setAttribute(NB_AGENTS, "" + nbNodes);
		for (int id = 1 ; id <= nbNodes ; id++) {
			Element agent = new Element(AGENT);
			agent.setAttribute(NAME, "A" + id);
			agent.setAttribute(ID, "" + id);
			agent.setAttribute(DESCRIPTION, NODE + id);
			agents.addContent(agent);
		}
		return agents;
	}

	public Element getDomaines(int nbColours) {
		Element domains = new Element(DOMAINS);
		domains.setAttribute(NB_DOMAINS, "1");
		Element domain = new Element("domain");
		domain.setAttribute(NAME, "D1");
		domain.setAttribute(NB_VALUES, "" + nbColours);
		domain.setText("0.." + (nbColours - 1));
		domains.addContent(domain);
		return domains;
	}

	public Element getVariables(int nbNodes) {
		Element variables = new Element(VARIABLES);
		variables.setAttribute(NB_VARIABLES, "" + nbNodes);
		for (int id = 1 ; id <= nbNodes ; id++) {
			Element variable = new Element(VARIABLE);
			variable.setAttribute(AGENT, "A" + id);
			variable.setAttribute(NAME, "X" + id + ".0");
			variable.setAttribute(ID, "0");
			variable.setAttribute(DOMAIN, "D1");
			variable.setAttribute(DESCRIPTION, "X" + id + ".0");
			variables.addContent(variable);
		}
		return variables;
	}

	private Element getRelations(int nbColours) {
		Element relations = new Element(RELATIONS);
		relations.setAttribute(NB_RELATIONS, "1");
		Element relation = new Element(RELATION);
		relation.setAttribute(NAME, "R1");
		relation.setAttribute(ARITY, "2");
		relation.setAttribute(NB_TUPLES, "" + nbColours);
		relation.setAttribute(SEMANTICS, CONFLICTS);
		String tuples = "";
		for (int value = 0 ; value < nbColours - 1 ; value++) {
			tuples += value + " " + value + "|";
		}
		tuples += (nbColours - 1) + " " + (nbColours - 1);
		relation.setText(tuples);
		relations.addContent(relation);
		return relations;
	}

	/**
	 * @param instance
	 * @param nbI
	 */
	private Element getConstraints(int nbNodes, int nbConstraints) {
		Element constraints = new Element(CONSTRAINTS);
		constraints.setAttribute(NB_CONSTRAINTS, "" + nbConstraints);
		Random rand = new Random();
		rand.setSeed(System.currentTimeMillis());
		/* CT means "constraint" */
		int PossibleCTs = nbNodes * (nbNodes - 1) / 2;
		/* Create an array for each possible binary constraint. */
		long CTarray[] = new long[PossibleCTs];
		long selectedCT;
		int i = 0;
		int var1, var2;

		/*
		 * Initialize the CTarray. Each entry has one var in the high two bytes, and the other in the low two bytes.
		 */
		for (var1 = 0 ; var1 < (nbNodes - 1) ; var1++) {
			for (var2 = var1 + 1 ; var2 < nbNodes ; var2++) {
				CTarray[i++] = (var1 << 16) | var2;
			}
		}
		/* Select this.nbConstraint constraints. */
		for (int c = 0 ; c < nbConstraints ; c++) {
			/* Choose a random number between c and PossibleCTs - 1, inclusive. */
			int r = c + (int) (rand.nextFloat() * (PossibleCTs - c));
			/* Swap elements [c] and [r]. */
			selectedCT = CTarray[r];
			CTarray[r] = CTarray[c];
			CTarray[c] = selectedCT;

			Element constraint = new Element(CONSTRAINT);
			constraint.setAttribute(NAME, "C" + (c + 1));
			constraint.setAttribute(MODEL, Commands.TKC);
			constraint.setAttribute(ARITY, "2");
			/* Broadcast the constraint. */
			var1 = (int) (CTarray[c] >> 16) + 1;
			var2 = (int) (CTarray[c] & 0x0000FFFF) + 1;
			// constraint.setAttribute(SCOPE_AGENTS, "A"+var1+" A"+var2);
			constraint.setAttribute(SCOPE, "X" + var1 + ".0" + " X" + var2 + ".0");
			constraint.setAttribute(REFERENCE, "R1");
			constraints.addContent(constraint);
		}
		return constraints;
	}

	public Element getInstance(int nbNodes, int nbColours, int nbConstraint, int numInstance) {
		Element instance = new Element(INSTANCE);
		instance.addContent(this.getPresentation(numInstance));
		instance.addContent(this.getAgents(nbNodes));
		instance.addContent(this.getDomaines(nbColours));
		instance.addContent(this.getVariables(nbNodes));
		instance.addContent(this.getRelations(nbColours));
		instance.addContent(this.getConstraints(nbNodes, nbConstraint));
		return instance;
	}

	/**
	 * 
	 */
	public void generate() {
		Element experiments = new Element(Commands.Experiments);
		Document confdocument = new Document(experiments);
		experiments.setAttribute(TYPE, Commands.GraphColoring);
		Element instances = new Element(INSTANCES);
		experiments.addContent(instances);
		instances.setAttribute(NAME, Commands.GC);
		int nbN = ((this.maxNbNodes - this.minNbNodes) / this.stepNbNodes + 1);
		int nbC = ((this.maxNbColours - this.minNbColours) / this.stepNbColours + 1);
		instances.setAttribute(NB_FOLDERS, "" + (nbN * nbC));
		// n=number of nodes , k=number of colours , c=number of constraint and
		// i= numeral of instance
		for (int n = this.minNbNodes ; n <= this.maxNbNodes ; n += this.stepNbNodes) {
			for (int k = this.minNbColours ; k <= this.maxNbColours ; k += this.stepNbColours) {
				File folder = new File(this.directory.getPath() + File.separator + n + "_" + k);
				if (!folder.isDirectory()) {
					folder.mkdir();
				}
				Element dir = new Element(DIRECTORY);
				instances.addContent(dir);
				dir.setAttribute(NB_NODES, "" + n);
				dir.setAttribute(NB_COLORES, "" + k);
				dir.setAttribute(NB_INSTANCES, "" + this.nbInstances);
				Element density = new Element(DENSITY);
				density.setAttribute(MIN_DENSITY, this.minDensity + "");
				density.setAttribute(MAX_DENSITY, this.maxDensity + "");
				density.setAttribute(STEP, this.stepDensity + "");
				dir.addContent(density);
				for (int c = this.minDensity ; c <= this.maxDensity ; c += this.stepDensity) {
					String d = (c < 10? "0.0" + c : "0." + c);
					File subfolder = new File(folder + File.separator + d);
					if (!subfolder.isDirectory()) {
						subfolder.mkdir();
					}
					int nbConstraint = (c * n * (n - 1)) / 200;
					for (int i = 1 ; i <= this.nbInstances ; i++) {
						Element instance = this.getInstance(n, k, nbConstraint, i);
						Document document = new Document(instance);
						String fileName = folder + File.separator + d + File.separator + Commands.GC + "_" + i + ".xml";
						this.save(document, fileName);
					}
				}
			}
		}
		this.saveInDirectory(confdocument, Commands.Experiments + "_" + this.nbInstances + ".xml");
	}

	@SuppressWarnings("unchecked")
	public static boolean lunchXP(File file) {
		int nbInstance, nbNodes, nbColours;
		String filePath = file.getParent();
		ArrayList<String> keys = null;
		// //////////////////////////////////////////////////////////////////////////////////////////////////////////
		Document document = null;
		SAXBuilder sxb = new SAXBuilder();
		try {
			document = sxb.build(file);
		} catch (Exception e) {}
		Element xp = document.getRootElement();
		Element instances = xp.getChild(INSTANCES);
		List<Element> communications = xp.getChild(Commands.Communications).getChildren();
		List<Element> algorithms = xp.getChild(Commands.Algorithms).getChildren();
		String[] protocols = new String[algorithms.size()];
		String[] orderings = new String[algorithms.size()];
		int a = 0;
		Iterator<Element> it = algorithms.iterator();
		while (it.hasNext()) {
			Element algorithm = it.next();
			protocols[a] = algorithm.getText();
			orderings[a] = algorithm.getAttributeValue(Commands.AO);
			a++;
		}
		it = communications.iterator();
		while (it.hasNext()) {
			// /////////////////////////////////////////////////////////////////
			Element comm = it.next();
			String commType = comm.getAttributeValue(TYPE);
			String channelType = comm.getAttributeValue(Commands.channelType);
			int minURC = new Integer(comm.getAttributeValue(Commands.MinURC));
			int maxURC = new Integer(comm.getAttributeValue(Commands.MaxURC));
			int nbRun = new Integer(comm.getAttributeValue(Commands.NbRun));
			long maxNCCCs = new Long(comm.getAttributeValue(Commands.MaxNCCCs));
			// /////////////////////////////////////////////////////////////////
			String name = instances.getAttribute(NAME).getValue();
			int nbFolders = new Integer(instances.getAttributeValue(NB_FOLDERS));
			List<Element> config = instances.getChildren();
			for (int f = 0 ; f < nbFolders ; f++) {
				Element dir = config.get(f);
				nbNodes = new Integer(dir.getAttributeValue(NB_NODES));
				nbColours = new Integer(dir.getAttributeValue(NB_COLORES));
				nbInstance = new Integer(dir.getAttributeValue(NB_INSTANCES));
				Element density = dir.getChild(DENSITY);
				String directory = filePath + File.separator + Commands.Results+"_"+nbInstance+"_"+nbRun;
				try {
					File d = new File(directory);
					if (!d.exists()) {
						d.mkdir();
					}
				} catch (Exception e) {}
				directory += File.separator + commType + "_" + channelType;
				try {
					File thefile = new File(directory);
					if (!thefile.exists()) {
						thefile.mkdir();
					}
				} catch (Exception e) {}
				String subDirectory = directory + File.separator + nbNodes + "_" + nbColours;
				try {
					File thefile = new File(subDirectory);
					if (!thefile.exists()) {
						thefile.mkdir();
					}
				} catch (Exception e) {}
				for (int p = 0 ; p < protocols.length ; p++) { // Protocols
					Logger allResults = new Logger(subDirectory, "All_" + protocols[p] + "-" + orderings[p] + ".csv");
					Logger results = new Logger(subDirectory, protocols[p] + "-" + orderings[p] + ".csv");
					int minDensity = new Integer(density.getAttributeValue(MIN_DENSITY));
					int maxDensity = new Integer(density.getAttributeValue(MAX_DENSITY));
					int stepDensity = new Integer(density.getAttributeValue(STEP));
					String path = filePath + File.separator + nbNodes + "_" + nbColours;
					for (int d = minDensity ; d <= maxDensity ; d += stepDensity) {
						String den = (d < 10? "0.0" + d : "0." + d);
						HashMap<String, Double> densityStatistics = new HashMap<String, Double>();
						for (int i = 1 ; i <= nbInstance ; i++) {
							String subFolder = (d < 10? "0.0" + d : "0." + d);
							String problem = path + File.separator + subFolder + File.separator + name + "_" + i + ".xml.bz2";
							HashMap<String, Double> instanceStatistics = new HashMap<String, Double>();
							for (int r = 1 ; r <= nbRun ; r++) {
								DisProblem disProb = new DisProblem(problem, protocols[p]);
								DisSolverSimulator solver = new DisSolverSimulator(disProb);
								solver.setCutOffNCCCs(maxNCCCs);
								solver.addCommunicationMetric();
								solver.setMessageDelay(minURC, maxURC);
								// solver.CheckTheReturnedSolution(true);
								solver.setCentralizedAO(AgentOrdering.getCentralizedAO(orderings[p]));
								solver.solve();
								if(keys == null ){
									keys = disProb.master.getStatisticsKeys();
								}
								HashMap<String, Double> runStatistics = solver.getStatistics();
								String s = ", ";
								if (d == minDensity && i == 1 && r == 1 && d == minDensity) {
									String header = Commands.GraphColoring + s;
									header += Commands.Algorithm + s + protocols[p] + s + Commands.AO + s + orderings[p] + "\n";
									header += NB_NODES + s + nbNodes + s + NB_COLORES + s + nbColours + "\n";
									header += Commands.channelType + s + channelType + s + Commands.Communication + s ;
									header += commType + s + Commands.MinURC + s + minURC + s + Commands.MaxURC + s + maxURC +  "\n\n";
									header += DENSITY + s + INSTANCE + s + Commands.NbRun + s + Tools.toString(keys);
									allResults.printToLogln(header);
									header = Commands.GraphColoring + s;
									header += Commands.Algorithm + s + protocols[p] + s + Commands.AO + s + orderings[p] + "\n";
									header += NB_NODES + s + nbNodes + s + NB_COLORES + s + nbColours + "\n";
									header += Commands.channelType + s + channelType + s + Commands.Communication + s + commType ;
									header += s + Commands.MinURC + s + minURC + s + Commands.MaxURC + s + maxURC + "\n\n";
									header += DENSITY + s + Tools.toString(keys);
									results.printToLogln(header);
								}
								allResults.printToLogln(den + s + i + s + r + s + Tools.statisticsToString(keys, runStatistics));
								instanceStatistics = Tools.merge(runStatistics, instanceStatistics);
								disProb.setAgents(null);
								System.gc();
							}
							Tools.average(instanceStatistics, nbRun);
							densityStatistics = Tools.merge(instanceStatistics, densityStatistics);
						}
						Tools.average(densityStatistics, nbInstance);
						results.printToLogln(den + ", " + Tools.statisticsToString(keys, densityStatistics));
					}
					results.close();
					allResults.close();
					keys = null;
				}
				String pathName = subDirectory;
				ResultPrinter printer = new ResultPrinter(protocols, orderings, nbHeaderLine);
				printer.setFixKeys(fixKeys);
				ArrayList<String> paths = printer.printToGnuPlot(pathName, "", commType);
				printer.plot(pathName, paths);
				//////
				String header = Commands.channelType + "," + channelType + "," + Commands.Communication + "," ; 
				header += commType + "," + Commands.MinURC + "," + minURC + "," + Commands.MaxURC + "," + maxURC;
				printer.printAllResults(pathName, header);
			}
		}
		return true;
	}

	/*
	 * (non-Javadoc)
	 * @see gui.Luncher#lunch()
	 */
	@Override
	@SuppressWarnings("unchecked")
	public void lunch() {
		int nbInstance, nbNodes, nbColours;
		String filePath = DisChoco.filePath.getParent();
		ArrayList<String> keys = null;
		String directory = filePath + File.separator + Commands.Results;
		try {
			File d = new File(directory);
			if (!d.exists()) {
				d.mkdir();
			}
		} catch (Exception e) {}
		// //////////////////////////////////////////////////////////////////////////////////////////////////////////
		Document document = null;
		SAXBuilder sxb = new SAXBuilder();
		try {
			document = sxb.build(DisChoco.filePath);
		} catch (Exception e) {}
		Element xp = document.getRootElement();
		Element instances = xp.getChild(INSTANCES);
		List<Element> communications = xp.getChild(Commands.Communications).getChildren();
		List<Element> algorithms = xp.getChild(Commands.Algorithms).getChildren();
		String[] protocols = new String[algorithms.size()];
		String[] orderings = new String[algorithms.size()];
		int a = 0;
		Iterator<Element> it = algorithms.iterator();
		while (it.hasNext()) {
			Element algorithm = it.next();
			protocols[a] = algorithm.getText();
			orderings[a] = algorithm.getAttributeValue(Commands.AO);
			a++;
		}
		it = communications.iterator();
		while (it.hasNext()) {
			// /////////////////////////////////////////////////////////////////
			Element comm = it.next();
			String commType = comm.getAttributeValue(TYPE);
			String channelType = comm.getAttributeValue(Commands.channelType);
			int minURC = new Integer(comm.getAttributeValue(Commands.MinURC));
			int maxURC = new Integer(comm.getAttributeValue(Commands.MaxURC));
			int nbRun = new Integer(comm.getAttributeValue(Commands.NbRun));
			long maxNCCCs = new Long(comm.getAttributeValue(Commands.MaxNCCCs));
			// /////////////////////////////////////////////////////////////////
			directory = filePath + File.separator + Commands.Results + File.separator + commType + "_" + channelType;
			try {
				File f = new File(directory);
				if (!f.exists()) {
					f.mkdir();
				}
			} catch (Exception e) {}
			String name = instances.getAttribute(NAME).getValue();
			int nbFolders = new Integer(instances.getAttributeValue(NB_FOLDERS));
			List<Element> config = instances.getChildren();
			for (int p = 0 ; p < protocols.length ; p++) { // Protocols
				for (int f = 0 ; f < nbFolders ; f++) {
					Element dir = config.get(f);
					nbNodes = new Integer(dir.getAttributeValue(NB_NODES));
					nbColours = new Integer(dir.getAttributeValue(NB_COLORES));
					Logger allResults =
							new Logger(directory + File.separator + nbNodes + "_" + nbColours, "All_" + protocols[p] + "-" + orderings[p] + ".csv");
					Logger results = new Logger(directory + File.separator + nbNodes + "_" + nbColours, protocols[p] + "-" + orderings[p] + ".csv");
					nbInstance = new Integer(dir.getAttributeValue(NB_INSTANCES));
					Element density = dir.getChild(DENSITY);
					int minDensity = new Integer(density.getAttributeValue(MIN_DENSITY));
					int maxDensity = new Integer(density.getAttributeValue(MAX_DENSITY));
					int stepDensity = new Integer(density.getAttributeValue(STEP));
					String path = filePath + File.separator + nbNodes + "_" + nbColours;
					for (int d = minDensity ; d <= maxDensity ; d += stepDensity) {
						String den = (d < 10? "0.0" + d : "0." + d);
						HashMap<String, Double> stat1 = new HashMap<String, Double>();
						DisSolverSimulator solver = null;
						for (int i = 1 ; i <= nbInstance ; i++) {
							String subFolder = (d < 10? "0.0" + d : "0." + d);
							DisProblem disProb = null;
							InstanceParser parser = new InstanceParser(name + "_" + i);
							String problem = path + File.separator + subFolder + File.separator + name + "_" + i + ".xml.bz2";
							parser.loadInstance(problem);
							parser.parse(false);
							HashMap<String, Double> stat2 = new HashMap<String, Double>();
							for (int r = 1 ; r <= nbRun ; r++) {
								AgentOrdering.getCentralizedAO(orderings[p]);
								disProb = new DisProblem(parser, protocols[p]);
								solver = new DisSolverSimulator(disProb);
								solver.setCutOffNCCCs(maxNCCCs);
								solver.solve();
								if(keys == null ){
									keys = disProb.master.getStatisticsKeys();
								}
								HashMap<String, Double> stat = solver.getStatistics();
								if (d == minDensity && i == 1 && r == 1 && d == minDensity) {
									String s = ",";
									String header = Commands.GraphColoring + s;
									header += Commands.Algorithm + s + protocols[p] + s + Commands.AO + s + orderings[p] + "\n";
									header += Commands.channelType + s + channelType + s + Commands.Communication + s + commType;
									header += s + Commands.MinURC + s + minURC + s + Commands.MaxURC + s + maxURC + "\n\n";
									header += NB_NODES + s + NB_COLORES + s + DENSITY + s + INSTANCE + s + Commands.NbRun + s + keys;
									allResults.printToLogln(header);
									header = Commands.GraphColoring + s;
									header += Commands.Algorithm + s + protocols[p] + s + Commands.AO + s + orderings[p] + "\n";
									header += Commands.channelType + s + channelType + s + Commands.Communication + s + commType ;
									header += s + Commands.MinURC + s + minURC + s + Commands.MaxURC + s + maxURC + "\n\n";
									header += DENSITY + s + keys;
									results.printToLogln(header);
								}
								stat2 = Tools.merge(stat, stat2);
								allResults.printToLogln(den + "," + i + "," + r + ","
										+ Tools.statisticsToString(keys, stat));
								solver = null;
								System.gc();
							}
							Tools.average(stat2, nbRun);
							stat1 = Tools.merge(stat2, stat1);
							parser = null;
						}
						Tools.average(stat1, nbInstance);
						results.printToLogln(den + "," + Tools.statisticsToString(keys, stat1));
					}
					results.close();
					allResults.close();
					keys = null;
				}
			}
			String header =
					Commands.channelType + "," + channelType + "," + Commands.Communication + "," + commType + "," + Commands.MinURC + "," + minURC
							+ "," + Commands.MaxURC + "," + maxURC;
			for (int f = 0 ; f < nbFolders ; f++) {
				Element dir = config.get(f);
				nbNodes = new Integer(dir.getAttributeValue(NB_NODES));
				nbColours = new Integer(dir.getAttributeValue(NB_COLORES));
				ResultPrinter printer = new ResultPrinter(protocols, orderings, nbHeaderLine);
				printer.setFixKeys(fixKeys);
				String subFolder = nbNodes + "_" + nbColours;
				ArrayList<String> paths = printer.printToGnuPlot(directory + File.separator + subFolder, "", commType);
				printer.plot(directory + File.separator + subFolder, paths);
				printer.printAllResults(directory + File.separator + subFolder, header);
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * @see gui.GuiSimulator#resumeWithoutSolver()
	 */
	@Override
	public void resumeWithoutSolver() {
		// TODO Auto-generated method stub
		System.err.println("I have to resume my experiments such that my solver is null");
	}
}
