package gui.generators;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.KeyEvent;
import java.io.File;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import benchmark.graphcoloring.gui.GCGeneratorPanel;
import benchmark.meetingscheduling.gui.MSGeneratorPanel;
import benchmark.nqueen.gui.NQueenGeneratorPanel;
import benchmark.random.gui.ComplexRandomTKCPanel;
import benchmark.random.gui.RandomDisCSPGeneratorPanel;
import benchmark.rmsp.gui.RandomMSPGeneratorPanel;
import benchmark.sensornetwork.gui.SensorNetworkGeneratorPanel;
import gui.LabelAccessory;
import util.Commands;
/**
 * 
 * 
 * 
 * @authors 
 * <a href="mailto:ezzahir@ensa-agadir.ac.ma?subject=DisChoco &cc=wahbi@lirmm.fr &cc=bessiere@lirmm.fr">
 * Redoaune Ezzahir</a> and 
 * <a href="mailto:wahbi@lirmm.fr?subject=DisChoco &cc=ezzahir@ensa-agadir.ac.ma &cc=bessiere@lirmm.fr">
 * Mohamed Wahbi</a>
 *
 */
public class GeneratorsPanel extends JPanel {

	public static final long serialVersionUID = 1L;

	private JPanel northPanel = null;
	private JPanel centerPanel = null;
	private JButton browse = null;
	private JButton generate = null;
	private JTabbedPane generatorsTabbedPane = null;
	// private GuiConsole guiConsole = null;
	// private JScrollPane consoleScrollPane = null;
	private JFileChooser fileChooser = null;
	private File file = null;  //  @jve:decl-index=0:

	private RandomDisCSPGeneratorPanel DisCSPPanel = null;
	private GCGeneratorPanel gcPanel = null;
	private MSGeneratorPanel mspPanel = null;
	private RandomMSPGeneratorPanel rmspPanel = null;
	private SensorNetworkGeneratorPanel snPanel = null;
	private NQueenGeneratorPanel nQueen = null;
	private ComplexRandomTKCPanel complexDisCSP = null;

	public static short randomDisCSP = 0;
	public static short graphColoring = 1;
	public static short meetingScheduling = 2;
	public static short sensorNetworks = 3;
	public static short nQueens = 4;
	public static short randomComplexDisCSP = 5;
	public static short randomMSP = 6;

	/**
	 * This is the default constructor
	 * 
	 * @param aPanel
	 * @param panel
	 * @param disChocoXP
	 */
	public GeneratorsPanel() {
		super();
		this.initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	public void initialize() {
		this.file = new File(".");
		getGraphColoringPanel();
		getRandomDisCSPPanel();
		getMeetingSchedulingPanel();
		getRandomMSPGeneratorPanel();
		getURSensorNetworksPanel();
		getNQueenPanel();
		getComplexRandomTKCPanel();
		this.setLayout(new BorderLayout());
		this.add(getNorthPanel(1024, 40), BorderLayout.NORTH);
		this.add(getGenerators(1024, 480), BorderLayout.CENTER);
		this.setMinimumSize(new Dimension(1024, 520));
		this.setMaximumSize(new Dimension(1024, 520));
		this.setPreferredSize(new Dimension(1024, 520));
		this.setBorder(BorderFactory.createEtchedBorder());
	}

	public void setGenerator(int generator) {
		getGeneratorsTabbedPane().setSelectedIndex(generator);
	}

	public GCGeneratorPanel getGraphColoringPanel() {
		if (gcPanel == null) {
			gcPanel = new GCGeneratorPanel();
		}
		return gcPanel;
	}

	public RandomDisCSPGeneratorPanel getRandomDisCSPPanel() {
		if (DisCSPPanel == null) {
			DisCSPPanel = new RandomDisCSPGeneratorPanel();
		}
		return DisCSPPanel;
	}

	public MSGeneratorPanel getMeetingSchedulingPanel() {
		if (mspPanel == null) {
			mspPanel = new MSGeneratorPanel();
		}
		return mspPanel;
	}

	public RandomMSPGeneratorPanel getRandomMSPGeneratorPanel() {
		if (rmspPanel == null) {
			rmspPanel = new RandomMSPGeneratorPanel();
		}
		return rmspPanel;
	}

	public SensorNetworkGeneratorPanel getURSensorNetworksPanel() {
		if (snPanel == null) {
			snPanel = new SensorNetworkGeneratorPanel();
		}
		return snPanel;
	}

	public NQueenGeneratorPanel getNQueenPanel() {
		if (nQueen == null) {
			nQueen = new NQueenGeneratorPanel();
		}
		return nQueen;
	}

	public ComplexRandomTKCPanel getComplexRandomTKCPanel() {
		if (complexDisCSP == null) {
			complexDisCSP = new ComplexRandomTKCPanel();
		}
		return complexDisCSP;
	}

	/**
	 * This method initializes northPanel
	 * 
	 * @param height
	 * @param width
	 * 
	 * @return javax.swing.JPanel
	 */
	public JPanel getNorthPanel(int width, int height) {
		if (northPanel == null) {
			northPanel = new JPanel();
			northPanel.setLayout(new FlowLayout());
			northPanel.setPreferredSize(new Dimension(width, height));
			northPanel.setMinimumSize(new Dimension(width, height));
			northPanel.add(getBrowser(), null);
			northPanel.add(getGenerate(), null);
		}
		return northPanel;
	}

	/**
	 * This method initializes jPanel3
	 * 
	 * @param height
	 * @param width
	 * 
	 * @return javax.swing.JPanel
	 */
	public JPanel getGenerators(int width, int height) {
		if (centerPanel == null) {
			centerPanel = new JPanel();
			centerPanel.setLayout(new BorderLayout());
			centerPanel.add(getGeneratorsTabbedPane(width, height), BorderLayout.CENTER);
		}
		return centerPanel;
	}

	/**
	 * This method initializes browse
	 * 
	 * @return javax.swing.JButton
	 */
	public JButton getBrowser() {
		if (browse == null) {
			browse = new JButton();
			browse.setText("Browse");
			browse.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					fileChooser = new JFileChooser(file);
					fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
//					fileChooser.setFileView(new JavaFileView());
					fileChooser.setAccessory(new LabelAccessory(fileChooser));
					int returnVal = fileChooser.showSaveDialog((Component) e.getSource());
					if (returnVal == JFileChooser.APPROVE_OPTION) {
						file = fileChooser.getCurrentDirectory();
						System.out.println("Path : "+file.getAbsolutePath() + "\n");
					}
				}
			});
		}
		return browse;
	}

	private JTabbedPane getGeneratorsTabbedPane() {
		if (generatorsTabbedPane == null) { return getGeneratorsTabbedPane(1024, 480); }
		return generatorsTabbedPane;
	}

	/**
	 * This method initializes generatorsTabbedPane
	 * 
	 * @param height
	 * @param width
	 * 
	 * @return javax.swing.JTabbedPane
	 */
	public JTabbedPane getGeneratorsTabbedPane(int width, int height) {
		if (generatorsTabbedPane == null) {
			generatorsTabbedPane = new JTabbedPane(JTabbedPane.TOP, JTabbedPane.SCROLL_TAB_LAYOUT);
			generatorsTabbedPane.addTab("Random DisCSP", null, getRandomDisCSPPanel(), Commands.URDisCSP + " generator");
			generatorsTabbedPane.setMnemonicAt(randomDisCSP, KeyEvent.VK_1);

			generatorsTabbedPane.addTab("Graph Coloring", null, getGraphColoringPanel(), Commands.URGColoring + " generator");
			generatorsTabbedPane.setMnemonicAt(graphColoring, KeyEvent.VK_2);

			generatorsTabbedPane.addTab("Meeting Scheduling", null, getMeetingSchedulingPanel(), Commands.DMSP + " generator");
			generatorsTabbedPane.setMnemonicAt(meetingScheduling, KeyEvent.VK_3);

			generatorsTabbedPane.addTab("Sensor Networks", null, getURSensorNetworksPanel(), Commands.DSNP + " generator");
			generatorsTabbedPane.setMnemonicAt(sensorNetworks, KeyEvent.VK_4);

			generatorsTabbedPane.addTab(Commands.DisNQueen, null, getNQueenPanel(), Commands.DisNQueen + " problem generator");
			generatorsTabbedPane.setMnemonicAt(nQueens, KeyEvent.VK_5);

			generatorsTabbedPane.addTab("Complex Random DisCSP", null, getComplexRandomTKCPanel(), Commands.URComplexDisCSP + " generator");
			generatorsTabbedPane.setMnemonicAt(randomComplexDisCSP, KeyEvent.VK_6);

			generatorsTabbedPane.addTab("Random MS", null, getRandomMSPGeneratorPanel(), Commands.RMSP + " generator");
			generatorsTabbedPane.setMnemonicAt(randomMSP, KeyEvent.VK_7);

			generatorsTabbedPane.setTabLayoutPolicy(JTabbedPane.WRAP_TAB_LAYOUT);
		}
		return generatorsTabbedPane;
	}

	/**
	 * This method initializes generate
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getGenerate() {
		if (generate == null) {
			generate = new JButton();
			generate.setText("Generate");
			generate.addActionListener(new java.awt.event.ActionListener() {
//				@SuppressWarnings("unchecked")
				public void actionPerformed(java.awt.event.ActionEvent e) {
					if (file == null) {
						JOptionPane.showMessageDialog(null, "Choose firstly file", "DisChoco", JOptionPane.WARNING_MESSAGE);
						return;
					}
					Generator generator = (Generator) getGeneratorsTabbedPane().getSelectedComponent();
					generator.generate(file);
					String ans = "problems are generated succefully";
					System.out.println(ans);
//					SwingWorker worker = new SwingWorker() {
//						@Override
//						protected Object doInBackground() throws Exception {
//							Generator generator = (Generator) getGeneratorsTabbedPane().getSelectedComponent();
//							generator.generate(file);
//							String ans = "problems are generated succefully";
//							System.out.println(ans);
//							return ans;
//						}
//					};
//					worker.execute();
				}
			});
		}
		return generate;
	}

}
