package simple.dcsp.agileabt.explanation;

import java.util.HashMap;
import java.util.Iterator;

import kernel.AbstractAgent;
import kernel.AbstractMaster;
import kernel.StopException;
import kernel.com.Message;
import kernel.com.TerminateMsgHandler;
import kernel.events.AgentEvent;
import simple.dcsp.abtfamily.IdleEvent;
import simple.dcsp.agileabt.StateOfAgent;
import util.BooleanLock;
import util.Instantiation;
import util.IntArray;
import util.Tools;
import util.VarAssignment;

public class AgileABTTermDetection extends TerminateMsgHandler {

	protected HashMap<Integer, StateOfAgent> stateOfAgents = null;
	protected BooleanLock whenIdleLock = new BooleanLock(false);
	protected StateOfAgent prevSentAccepted = null;
	protected IntArray lastReceivers;
	protected IntArray bestOrder;
	protected double[] bestTV = null;

	public AgileABTTermDetection(AbstractAgent agent) {
		super(agent);
		this.stateOfAgents = new HashMap<Integer, StateOfAgent>();
	}

	public void initialize(IntArray order, double[] tv) {
		this.stateOfAgents.put(this.myAgent.agentId, new StateOfAgent(order, tv));
		this.SetBestOrder(order, tv);
		this.prevSentAccepted = null;
		this.lastReceivers = new IntArray();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.TerminateMsgHandler#processTerminationDetection(.com.Message)
	 */
	public boolean processTerminationDetection(Message msg) throws StopException {
		int senderId = msg.getSenderId();
		StateOfAgent receivedState = (StateOfAgent) msg.get(AgileABTAgent.State);
		this.setState(senderId, receivedState);
		IntArray lowerNeighbors = this.myAgent.communicator.getLowerNeighbors();
		lowerNeighbors = IntArray.intersection(lowerNeighbors, this.myAgent.communicator.getNeighbors0());
		if (lowerNeighbors.size() == this.stateOfAgents.size()) {
			this.tryToSendAccepted();
		}
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.TerminateMsgHandler#handleAgentEvent(events.AgentEvent )
	 */

	@Override
	public void handleAgentEvent(AgentEvent agentEvent) {
		if (agentEvent instanceof IdleEvent && !this.myAgent.isFinished()) {
			this.tryToSendAccepted();
		}
		if (agentEvent instanceof kernel.events.AgentFinishedEvent) {
			this.askMasterToTerminate(true);
		}
	}

	/**
	 * 
	 * @param agId
	 * @param receivedState
	 */
	public void setState(int agId, StateOfAgent receivedState) {
		IntArray order = receivedState.getOrder();
		double[] tv = receivedState.getTV();
		if (StateOfAgent.areAcceptable(this.bestOrder, this.bestTV, order, tv) < 0) {
			this.SetBestOrder(order, tv);
		}
		if (StateOfAgent.areAcceptable(this.bestOrder, this.bestTV, order, tv) == 0) {
			this.stateOfAgents.put(agId, receivedState);
		}
	}

	/**
	 * 
	 * @param order
	 * @param tv
	 */
	public void SetBestOrder(IntArray order, double[] tv) {
		this.bestOrder = order;
		this.bestTV = tv;
		Iterator<Integer> it = this.stateOfAgents.keySet().iterator();
		while (it.hasNext()) {
			int agId = it.next();
			StateOfAgent state = this.stateOfAgents.get(agId);
			if (StateOfAgent.areAcceptable(this.bestOrder, this.bestTV, state.getOrder(), state.getTV()) > 0) {
				it.remove();
			}
		}
	}

	/**
	 * 
	 */
	public void tryToSendAccepted() {
		// if(Logger.LOG_DEBUG){
		// this.myAgent.log("I try to send an accepted message : ",1);
		// }
		StateOfAgent accepted = this.buildAccepted();
		if (((AgileABTAgent) this.myAgent).isIdle() && accepted != null) {
			if (this.IsendTheSame(accepted)) {
				// if(Logger.LOG_DEBUG){
				// this.myAgent.log("I have sent the same accepted befor: "+accepted,1);
				// }
				return;
			}
			this.prevSentAccepted = accepted;
			this.lastReceivers = new IntArray();
			IntArray higherNeighbors = this.myAgent.communicator.getHigherNeighbors();
			higherNeighbors = IntArray.intersection(higherNeighbors, this.myAgent.communicator.getNeighbors0());
			if (higherNeighbors.size() == 0) {
				this.lastReceivers.add(AbstractMaster.MasterId);
				this.sendAccepted(accepted, AbstractMaster.MasterId);
			} else {
				this.lastReceivers = higherNeighbors.clone();
				for (int highNeig : higherNeighbors) {
					this.sendAccepted(accepted, highNeig);
				}
			}
		}
	}

	/**
	 * 
	 * @param order
	 * @param tv
	 * @param agentView
	 */
	public void SetMyState(IntArray order, double[] tv, Instantiation agentView) {
		StateOfAgent myState = new StateOfAgent(order, tv, agentView);
		this.setState(this.myAgent.agentId, myState);
	}

	/**
	 * 
	 * @param accepted
	 * @return
	 */
	private boolean IsendTheSame(StateOfAgent accepted) {
		boolean samePrev = this.prevSentAccepted != null && this.prevSentAccepted.equals(accepted);
		IntArray higherNeighbors = this.myAgent.communicator.getHigherNeighbors();
		higherNeighbors = IntArray.intersection(higherNeighbors, this.myAgent.communicator.getNeighbors0());
		boolean sameReceiver = false;
		if (higherNeighbors.size() == 0) {
			sameReceiver = this.lastReceivers.contains(AbstractMaster.MasterId) && this.lastReceivers.size() == 1;
		} else {
			sameReceiver = this.lastReceivers.equals(higherNeighbors);
		}
		return (samePrev && sameReceiver);
	}

	/**
	 * 
	 * @return
	 */
	protected synchronized StateOfAgent buildAccepted() {
		if (!this.stateOfAgents.containsKey(this.myAgent.agentId)) {
			if (!((AgileABTAgent) this.myAgent).isIdle()) {
				// if (Logger.LOG_DEBUG) {
				// this.myAgent.log("I am not idle", 1);
				// }
				return null;
			}
			Instantiation myView = this.myAgent.agentView.getContextsOfHigherAgents();
			VarAssignment myAss = ((AgileABTAgent) this.myAgent).var.getVarAssignment();
			myView.add(myAss);
			IntArray myOrder = ((AgileABTAgent) this.myAgent).getOrder();
			double[] myTV = ((AgileABTAgent) this.myAgent).getMyTV();
			this.SetMyState(myOrder, myTV, myView);
			if (!this.stateOfAgents.containsKey(this.myAgent.agentId)) {
				// if (Logger.LOG_DEBUG) {
				// this.myAgent.log("my State is very old: "+myOrder+"@"+Utility.arrayToString(myTV)+" ==> "+myView,1);
				// }
				return null;
			}
		}
		StateOfAgent accepted = this.stateOfAgents.get(this.myAgent.agentId).clone();
		IntArray lowerNeighbors = this.myAgent.communicator.getLowerNeighbors();
		lowerNeighbors = IntArray.intersection(lowerNeighbors, this.myAgent.communicator.getNeighbors0());
		for (int lower : lowerNeighbors) {
			if (!this.stateOfAgents.containsKey(lower)) {
				// if(Logger.LOG_DEBUG){
				// this.myAgent.log("I do not receive an accepted from : "+lower+"\n"+this.toString(),1);
				// }
				return null;
			}
			StateOfAgent stateOfAgent = this.stateOfAgents.get(lower);
			if (accepted.isCompatible(stateOfAgent)) {
				accepted.priorityMerge(stateOfAgent.getAgentView());
			} else {
				// if(Logger.LOG_DEBUG){
				// this.myAgent.log("myState : "+accepted+"\nis incompatible with the state of "+lower+":\n"+stateOfAgent,1);
				// }
				return null;
			}
		}
		return accepted;
	}

	/**
	 * 
	 * @param accepted
	 * @param receiverId
	 */
	protected void sendAccepted(StateOfAgent accepted, int receiverId) {
		Message msg = this.createMessage();
		msg.addContents(AgileABTAgent.State, accepted);
		msg.setReceiverId(receiverId);
		this.myAgent.sendMessage(msg);
		// if(Logger.LOG_DEBUG){
		// this.myAgent.log("send Message : "+msg,1);
		// }
	}

	/**
	 * 
	 */
	public String toString() {
		String ans = "\nOrder ==> \n" + this.bestOrder + "@" + Tools.arrayToString(this.bestTV) + "\n";
		Iterator<Integer> it = this.stateOfAgents.keySet().iterator();
		while (it.hasNext()) {
			int agId = it.next();
			StateOfAgent state = this.stateOfAgents.get(agId);
			ans += "" + agId + "=" + state.toString() + "\n";
		}
		return ans;
	}
}
