package simple.dcsp.agileabt.explanation;

import java.util.HashMap;
import java.util.Iterator;

import kernel.MasterObserver;
import kernel.com.Message;
import simple.dcsp.agileabt.StateOfAgent;
import util.Commands;
import util.Instantiation;
import util.IntArray;
import util.Logger;
import util.Tools;

/**
 * termination detection (Silaghi et al. 2000)
 * 
 * @author redouane
 * 
 */
public class TerminationDetector extends MasterObserver {

	protected HashMap<Integer, Instantiation> acceptedMsg;
	protected HashMap<Integer, StateOfAgent> stateOfAgents = null;

	protected IntArray initialEndAgents = new IntArray();
	protected IntArray order = null;
	protected double[] tv = null;

	public TerminationDetector() {
		super();
		this.acceptedMsg = new HashMap<Integer, Instantiation>();
		this.stateOfAgents = new HashMap<Integer, StateOfAgent>();
	}

	/**
	 * 
	 * @param order
	 * @return
	 */
	private double[] getMaximumTV(IntArray order) {
		double[] tv = new double[order.size()];
		for (int i = 0 ; i < tv.length ; i++) {
			tv[i] = Integer.MAX_VALUE;
		}
		return tv;
	}

	/**
	 * 
	 * @param order
	 */
	public void setInitialOrder(IntArray order) {
		if (this.order == null) {
			this.order = order;
			this.tv = this.getMaximumTV(order);
		}
	}

	private void ProcessOrder(Message msg) {
		IntArray order = (IntArray) msg.get(AgileABTAgent.Order);
		double[] TV = Tools.DoubleArrayFromString("" + msg.get(AgileABTAgent.TV));
		this.setInitialOrder(order);
		if (StateOfAgent.areAcceptable(this.order, this.tv, order, tv) < 0) {
			this.order = order;
			this.tv = TV;
			this.computeInitialEndAgents();
			this.checkEnd();
		}
		if (Logger.LOG_DEBUG) {
			this.master.log("IEA: " + this.initialEndAgents + " ==>\n  " + this.order + "@" + Tools.arrayToString(this.tv) + "\n"
					+ this.stateOfAgents + "\n receiveMsg : " + msg + "\n" + this.toString(), 1);
		}
	}

	@Override
	public boolean handleReceivedMessage(Message msg) {
		if (msg.getType().equals(AgileABTAgent.Order)) {
			this.ProcessOrder(msg);
		}
		if (msg.getType().equals(Commands.TERMINATION_DETETION)) {
			int senderId = msg.getSenderId();
			StateOfAgent receivedState = (StateOfAgent) msg.get(AgileABTAgent.State);
			IntArray order = receivedState.getOrder();
			this.setInitialOrder(order);
			double[] tv = receivedState.getTV();
			int acceptable = StateOfAgent.areAcceptable(this.order, this.tv, order, tv);
			if (acceptable < 0) {
				this.order = order;
				this.tv = tv;
				this.stateOfAgents.put(senderId, receivedState);
			} else {
				if (acceptable == 0) {
					if (this.stateOfAgents.keySet().contains(senderId)) {
						StateOfAgent stateOfSender = this.stateOfAgents.get(senderId);
						stateOfSender.setAgentView(order, tv, receivedState.getAgentView());
					} else {
						this.stateOfAgents.put(senderId, receivedState);
					}
				}
			}
			this.checkEnd();
		}
		return true;
	}

	private IntArray GammaPlus(int agId) {
		IntArray higherAgents = this.order.getElementsBefor(agId);
		IntArray neighbours = this.master.getNeighbors(agId);
		return IntArray.intersection(higherAgents, neighbours);
	}

	private IntArray computeInitialEndAgents() {
		this.initialEndAgents = new IntArray();
		for (int agId : this.order) {
			if (this.GammaPlus(agId).size() == 0) {
				this.initialEndAgents.add(agId);
			}
		}
		return this.initialEndAgents;
	}

	private StateOfAgent allReceivedAndAllCompatible() {
		StateOfAgent globalState = new StateOfAgent(this.order, this.tv);
		for (int agId : this.initialEndAgents) {
			if (!this.stateOfAgents.keySet().contains(agId)) {
				if (Logger.LOG_DEBUG) {
					this.master.log("accepted not yet received from : " + agId, 1);
				}
				return null;
			}
			StateOfAgent state = this.stateOfAgents.get(agId);
			if (!globalState.isCompatible(state)) {
				if (Logger.LOG_DEBUG) {
					this.master.log("Incompatible state of : " + agId + "=" + state + " with the globale one:\n" + globalState, 1);
				}
				return null;
			}
			globalState.priorityMerge(state.getAgentView());
		}
		return globalState;
	}

	public void removeIncompatibleState() {
		Iterator<Integer> it = this.stateOfAgents.keySet().iterator();
		while (it.hasNext()) {
			int agId = it.next();
			StateOfAgent state = this.stateOfAgents.get(agId);
			int acceptable = StateOfAgent.areAcceptable(this.order, this.tv, state.getOrder(), state.getTV());
			if (acceptable > 0) {
				it.remove();
			}
		}
	}

	private boolean checkEnd() {
		this.removeIncompatibleState();
		this.computeInitialEndAgents();
		if (this.stateOfAgents.size() < this.initialEndAgents.size()) {
			if (Logger.LOG_DEBUG) {
				this.master.log("\n stateOfAgents size : " + this.stateOfAgents.size() + "initialEndAgents size" + this.initialEndAgents.size(), 1);
			}
			return false;
		}
		StateOfAgent solution = this.allReceivedAndAllCompatible();
		if (solution == null) {
			if (Logger.LOG_DEBUG) {
				this.master.log("non compatible accepted " + solution + " \n", 1);
			}
			return false;
		}
		if (Logger.LOG_DEBUG) {
			this.master.log("terminate with SUCCESS : " + solution, 1);
		}
		this.master.setEndState(Commands.SUCCESS);
		this.master.broadcastSystemMessage(0);
		return true;
	}

	@Override
	public void handleSendMessage(Message msg) {}

	public String toString() {
		String ans = "\nIEAgents : " + this.initialEndAgents + " ==> " + this.order + "@" + Tools.arrayToString(this.tv) + "\n";
		Iterator<Integer> it = this.stateOfAgents.keySet().iterator();
		while (it.hasNext()) {
			int agId = it.next();
			StateOfAgent state = this.stateOfAgents.get(agId);
			ans += "" + agId + "=" + state.toString() + "\n";
		}
		return ans;
	}
}
