package util;

/**
 * Implements utilities for handling bit sets
 */
public class BitSet {

	/**
	 * tests whether the i-th bit of an search is on or off
	 */
	public static boolean getBit(int n, int i) {
		return (((n >> i) & 1) == 1);
	}

	/**
	 * sets the i-th bit of an search (does nothing if it is already on)
	 */
	public static int setBit(int n, int i) {
		if (!getBit(n, i))
			n += (1 << i);
		return n;
	}

	/**
	 * sets the i-th bit of an search (does nothing if it is already on)
	 */
	public static int unsetBit(int n, int i) {
		if (getBit(n, i)) {
			n -= (1 << i);
		}
		return n;
	}

	/**
	 * returns the heaviest bit on (-1 if none)
	 */
	public static int getHeavierBit(int n) {
		for (int i = 31 ; i >= 0 ; i--) {
			if (getBit(n, i))
				return i;
		}
		return -1;
	}

	/**
	 * i inclusive
	 * 
	 * @param n
	 * @param i
	 */
	public static int lowestBit(int n, int i) {
		for (int j = i ; j <= 31 ; j++) {
			if (getBit(n, j))
				return j;
		}
		return 32;
	}

	public static int lowestBit(int n) {
		return lowestBit(n, 0);
	}

	public static String toBitString(int n) {
		int tab[] = new int[32];
		for (int i = 0 ; i < 32 ; i++) {
			if (getBit(n, i))
				tab[31 - i] = 1;
		}
		return Tools.arrayToString(tab);
	}

	public static void main(String arg[]) {
		for (int i = 0 ; i < 10 ; i++) {
			System.err.println(toBitString(i) + " Heavier " + getHeavierBit(i) + " left " + lowestBit(i));
			System.err.println(BitSet.unsetBit(i, lowestBit(i)));
		}
	}

}
