package util.parser;

import java.util.ArrayList;

import kernel.AbstractAgent;
import kernel.Constraint;
import kernel.VarIdentifier;
import kernel.vars.integer.IntExternalVar;
import simple.SimpleAgent;
import util.Commands;

public class PExtensionConstraint extends PConstraint {

	private PRelation relation;

	public PRelation getRelation() {
		return relation;
	}

	public PExtensionConstraint(String name, PVariable[] scope, PRelation relation) {
		super(name, scope);
		this.relation = relation;
	}

	public int getMaximalCost() {
		return relation.getMaximalCost();
	}

	public long computeCostOf(int[] tuple) {
		return relation.computeCostOf(tuple);
	}

	public String toString() {
		return super.toString() + " reference=\"" + relation.getName() + "\"/>";
	}

	/**
	 * 
	 */
	public void makeConstraints() {
		int arity = this.relation.getArity();
		if (arity == 2) {
			PAgent[] agents = this.getScopeAgent();
			for (int i = 0 ; i < agents.length ; i++) {
				PAgent ag = agents[i];
				AbstractAgent gent = ag.getDisChocoAgent();
				if (InstanceParser.constraintModel.equals(Commands.PKC)) {
					if (gent != null && gent.getAgentId() == this.pkcOwnerId) {
						this.makeConstraint(gent, i);
						return;
					}
				} else if (gent != null) {
					this.makeConstraint(gent, i);
				}
			}
		} else
			throw new UnsupportedOperationException("");
	}

	private void makeConstraint(AbstractAgent agent, int agIdx) {
		if (agent instanceof SimpleAgent) {
			this.makeSConstraint((SimpleAgent) agent, agIdx);
		} else
			; // TODO
	}

	private void makeSConstraint(SimpleAgent agent, int agIdx) {
		int[][] tuples = this.relation.getTuples();
		String semantic = this.relation.getSemantics();
		boolean feas = false;
		if (semantic.equals(InstanceTokens.SUPPORTS))
			feas = true;
		ArrayList<int[]> couples = new ArrayList<int[]>();
		for (int i = 0 ; i < this.relation.getNbTuples() ; i++) {
			int[] tuple = new int[2];
			tuple[agIdx] = tuples[i][0];
			tuple[1 - agIdx] = tuples[i][1];
			couples.add(tuple);
		}
		int vId = this.getScope()[1 - agIdx].getId();
		int agId = this.getScope()[1 - agIdx].getAgent().getId();
		VarIdentifier extVarId = new VarIdentifier(agId, vId);
		int[] values = this.getScope()[1 - agIdx].getDomain().getValues();
		IntExternalVar extVar = agent.makeExternalVar(extVarId, values);
		Constraint exC = null;
		if (this.relation instanceof PSoftRelation) {
			// int[] costs = ((PSoftRelation) this.relation).getWeights();
			// if (feas)
			// exC = agent.feasiblePair(extVar, couples, costs);
			// else
			// exC = agent.infeasiblePair(extVar, couples, costs);
		} else {
			if (feas)
				exC = agent.feasiblePair(extVar, couples);
			else
				exC = agent.infeasiblePair(extVar, couples);

		}
		exC.setReference(this.name);
	}

}
